from humpday.optimizers.alloptimizers import OPTIMIZERS
from humpday.objectives.allobjectives import OBJECTIVES
from humpday.objectives.allobjectives import CLASSIC_OBJECTIVES
from humpday.comparison.suggestions import suggest, recommend
from humpday.optimizers.meta import minimize
from humpday.comparison.odious import compare, points_race

