from humpday.objectives.classic import CLASSIC_OBJECTIVES, A_CLASSIC_OBJECTIVE
from humpday.objectives.portfolio import PORTFOLIO_OBJECTIVES
from humpday.objectives.horse import HORSE_OBJECTIVES

OBJECTIVES = CLASSIC_OBJECTIVES + PORTFOLIO_OBJECTIVES + HORSE_OBJECTIVES
